/* Game.cpp
Defines the Game class
(C) 2000 N. Douglas
18th April 2000
*/

#include "Game.h"
#include <stdio.h>
#include "resource.h"

static DWORD WINAPI threadexeStub(LPVOID par)
{
	Game *g;
	g=(Game *) par;
	return (DWORD) g->threadexe();
}

static BOOL CALLBACK ToolboxDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{ 
    switch (uMsg)
	{ 
	case WM_INITDIALOG:
		{
			SetWindowLong(hwndDlg, DWL_USER, (LONG) lParam);
			SetFocus(GetDlgItem(hwndDlg, IDC_TBPAR1));
			return FALSE;
		}
	case WM_WINDOWPOSCHANGING:
		{
			LPWINDOWPOS wp=(LPWINDOWPOS) lParam;
			if(!(wp->flags & SWP_NOMOVE))
			{
				Game *g=(Game *) GetWindowLong(hwndDlg, DWL_USER);
				g->alignToolbox(wp, true);
			}
			return TRUE;
		}
	case WM_COMMAND:
		{
			// See if it's an accelerator generated message
			if(HIWORD(wParam)==1)
			{
				// Send it to main window
				Game *g=(Game *) GetWindowLong(hwndDlg, DWL_USER);
				SetForegroundWindow(g->hwndMain);
				SendMessage(g->hwndMain, uMsg, wParam, lParam);
				break;
			}
			if(HIWORD(wParam)==BN_CLICKED)
			{
				Game *g=(Game *) GetWindowLong(hwndDlg, DWL_USER);
				if(LOWORD(wParam)==IDC_TBPAUSED)
					SendMessage(g->hwndMain, WM_COMMAND, IDM_PAUSEGAME, 0);
				if(LOWORD(wParam)==IDC_TBLIQDENSON)
					SendMessage(g->hwndMain, WM_COMMAND, IDM_WEIGHWATER, 0);
				if(LOWORD(wParam)==IDC_TBFILTERON)
					SendMessage(g->hwndMain, WM_COMMAND, IDM_FILTERON, 0);
				g->updateToolbox();
				break;
			}
			break;
		}
    } 
    return FALSE; 
} 
 
static BOOL CALLBACK MolNoDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{ 
    switch (uMsg)
	{ 
	case WM_INITDIALOG:
		{
			Game *g;
			g=(Game *) lParam;
			TEST_WIN32ERR(!SetDlgItemInt(hwndDlg, IDC_NOOFMOLS, g->we->mols, FALSE));
			TEST_WIN32ERR(!CheckDlgButton(hwndDlg, IDC_SETNOFROMFPS, g->automolno ? BST_CHECKED : BST_UNCHECKED));
			TEST_WIN32ERR(!CheckRadioButton(hwndDlg, IDC_SNM_NORMAL, IDC_SNM_MIXED, IDC_SNM_MIXED));
			PostMessage(hwndDlg, WM_COMMAND, IDC_SETNOFROMFPS, 0);
			SetFocus(GetDlgItem(hwndDlg, IDC_NOOFMOLS));
			return TRUE;
		}
        case WM_COMMAND: 
            switch (LOWORD(wParam))
			{ 
                case IDC_SETMOLNO:
				{
					PostMessage(NULL, WM_COMMAND, IDC_SETMOLNO, 1);
					return TRUE;
				}
				case IDCANCEL:
				{
					PostMessage(NULL, WM_COMMAND, IDC_SETMOLNO, 0);
                    return TRUE; 
				}
				case IDC_SETNOFROMFPS:
				{
					BOOL en;
					en=IsDlgButtonChecked(hwndDlg, IDC_SETNOFROMFPS)!=BST_CHECKED;
					EnableWindow(GetDlgItem(hwndDlg, IDC_NOOFMOLSTXT), en);
					EnableWindow(GetDlgItem(hwndDlg, IDC_NOOFMOLS), en);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SNM_COLTXT), en);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SNM_NORMAL), en);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SNM_LIGHT), en);
					EnableWindow(GetDlgItem(hwndDlg, IDC_SNM_MIXED), en);
					return TRUE;
				}
            }
			break;
    } 
    return FALSE; 
} 
 
static BOOL CALLBACK InfoDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{ 
    switch (uMsg)
	{ 
	case WM_INITDIALOG:
		{
			TEST_WIN32ERR(!SetDlgItemText(hwndDlg, IDC_VERSION, PROGVERSION));
			TEST_WIN32ERR(!SetDlgItemText(hwndDlg, IDC_DATE, PROGDATE));
			TEST_WIN32ERR(!SetDlgItemText(hwndDlg, IDC_REGEDTO, licencee));
			return TRUE;
		}
	case WM_LBUTTONDOWN:
	case WM_MOUSEMOVE:
		{
			POINT p;
			HWND fhwnd;
			p.x=LOWORD(lParam);
			p.y=HIWORD(lParam);
			fhwnd=ChildWindowFromPoint(hwndDlg, p);
			if(fhwnd==GetDlgItem(hwndDlg, IDC_GOFLOWWWW) ||
				fhwnd==GetDlgItem(hwndDlg, IDC_GOFLOWWWW2))
			{
				SetCursor(LoadCursor((HINSTANCE) GetWindowLong(hwndDlg, GWL_HINSTANCE), MAKEINTRESOURCE(IDC_CURSORHAND)));
				if(uMsg==WM_LBUTTONDOWN)
				{
					PostMessage(hwndDlg, WM_COMMAND, IDC_GOFLOWWWW, 0);
				}
				return TRUE;
			}
			break;
		}
        case WM_COMMAND: 
            switch (LOWORD(wParam))
			{ 
			case IDC_GOFLOWWWW2:
				wParam=IDC_GOFLOWWWW;
				// Fall through
				case IDC_GOFLOWWWW:
                case IDD_GOTOHELP:
				case IDCANCEL:
                case IDOK: 
                    EndDialog(hwndDlg, wParam); 
                    return TRUE; 
            } 
    } 
    return FALSE; 
}

static BOOL CALLBACK PrefsMolColDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static Bitmap *colbmp=NULL;
	static u32 NormalCol, LightCol;

	switch(uMsg)
	{
	case WM_INITDIALOG:
		{
			Game *g;
			g=(Game *) lParam;
			TEST_WIN32ERR(!CheckDlgButton(hwndDlg, IDC_PREFS_NORMAL, BST_CHECKED));
			if(!colbmp) colbmp=new ScreenImage8(hwndDlg, 160, 160);
			// Set up the colour choice box
			{
				char *addr;
				u32 col=10-1;
				int x,y;
				addr=(char *) colbmp->getAddr();
				// Need to do 10...240 losing the six magic cols on the way
				// Each colour takes 10 pixels, 8 of colour and 1 each side for border
				for(y=10; y<160; y+=10)
				{
					for(x=0; x<160; x+=10)
					{
						col++;
						// Exclusion rules
						if(col>245) continue;
						if(col==g->we->pumpUscrcol) continue;
						if(col==g->we->pumpLscrcol) continue;
						if(col==g->we->pumpRscrcol) continue;
						if(col==g->we->filtAscrcol) continue;
						if(col==g->we->filtRscrcol) continue;
						if(col==g->we->filtBscrcol) continue;

						for(int y2=1; y2<9; y2++)
						{
							for(int x2=1; x2<9; x2++)
								addr[(y+y2)*160+(x+x2)]=(char) col;
						}
					}
				}
				colbmp->releaseAddr();
				// Set chosen colours to whatever prefs say they are
				NormalCol=g->bmp->RGBtoPhysical(g->prefs.normalmolcol);
				LightCol=g->bmp->RGBtoPhysical(g->prefs.lightmolcol);
				PostMessage(hwndDlg, WM_COMMAND, IDC_PREFS_NORMAL, 0);
			}

			return TRUE;
		}
	case WM_DESTROY:
		{
			// Kill the DirectDraw surface
			if(colbmp)
			{
				delete colbmp;
				colbmp=NULL;
			}
			break;
		}
	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC dc;
			RECT rFrame;
			TEST_WIN32ERR(!GetWindowRect(GetDlgItem(hwndDlg, IDC_PREFS_COLBOX), &rFrame));
			dc=BeginPaint(hwndDlg, &ps);
			colbmp->paint(dc, rFrame.left+1, rFrame.top+1, 1);
			EndPaint(hwndDlg, &ps);
			return 0;
		}
	case WM_COMMAND:
		{
			POINT p;
			RECT rFrame;
			char col=0;
			TEST_WIN32ERR(!GetWindowRect(GetDlgItem(hwndDlg, IDC_PREFS_COLBOX), &rFrame));

		switch(LOWORD(wParam))
		{
		case IDC_PREFS_COLBOX:
			{
				int x,y;
				char *addr;
				// Set Normal or Light to whatever he clicked on
				GetCursorPos(&p);
				x=p.x-rFrame.left;
				y=p.y-rFrame.top;
				addr=(char *) colbmp->getAddr();
				col=addr[y*160+x];
				if(!col) break;
				if(BST_CHECKED==IsDlgButtonChecked(hwndDlg, IDC_PREFS_NORMAL)) NormalCol=col;
				if(BST_CHECKED==IsDlgButtonChecked(hwndDlg, IDC_PREFS_LIGHT)) LightCol=col;
				// Fall through
			}
		case IDC_PREFS_DEFAULT:
			{
				// Return to default settings
				if(!col)
				{
					NormalCol=0x03;
					LightCol=0xe0;
				}
				// Invalidate the area which gets updated in the next bit of code
				p.x=rFrame.left;		p.y=rFrame.top;
				ScreenToClient(hwndDlg, &p);
				rFrame.left=p.x;		rFrame.top=p.y;
				p.x=rFrame.right;		p.y=rFrame.bottom;
				ScreenToClient(hwndDlg, &p);
				rFrame.right=p.x;		rFrame.bottom=p.y;
				InvalidateRect(hwndDlg, &rFrame, FALSE);
				// Fall through
			}
		case IDC_PREFS_NORMAL:
		case IDC_PREFS_LIGHT:
			{
				char *addr;
				int x,y;
				addr=(char *) colbmp->getAddr();
				// Update currently chosen selections
				for(y=1; y<9; y++)
				{
					for(x=1; x<79; x++)
						addr[y*160+x]=(char) NormalCol;
					for(x=81; x<159; x++)
						addr[y*160+x]=(char) LightCol;
				}
				colbmp->releaseAddr();
				break;
			}
		case IDCANCEL:
			{
				PostMessage(NULL, WM_COMMAND, IDC_PREFS_MOLCOLS, 0);
				return TRUE;
			}
		case IDC_PREFS_MOLCOLS:
		case IDOK:
			{
				PostMessage(NULL, WM_COMMAND, IDC_PREFS_MOLCOLS, (LightCol<<8)+NormalCol);
				return TRUE;
			}
		}
		}
	}
	return FALSE;
}

static BOOL CALLBACK CalibrDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{ 
    return FALSE; 
} 
 
Game::Game(HINSTANCE hInstance, DWORD parentthreadid, char *_name, int _nCmdShow)
{
    hinst = hInstance;  // save instance handle 
	strcpy(name, _name);
	timeofcreation=GetTickCount();
	bmpscale=1;
	drawing=false;
	paused=true;
	lockX=lockY=false;
	parentthreadID=parentthreadid;
	nCmdShow=_nCmdShow;
	tool=NULL;
	hwndToolbox=hwndMolNo=hwndMolCol=hwndCalibr=NULL;
	automolno=true;
	fps[0]=fps[1]=fps[2]=fps[3]=fps[4]=0;
	fpsidx=-1;

	// Load keyboard shortcuts
	TEST_WIN32ERR((hacclMain=LoadAccelerators(hinst, MAKEINTRESOURCE(IDR_MAINMENUACC)))==NULL);

	// Create a thread to run this particular game
	mythreadH=CreateThread(NULL, 0, threadexeStub, (LPVOID) this, 0, &mythreadID);
	TEST_WIN32ERR(mythreadH==NULL);
}

Game::~Game()
{
	final();
}

void Game::final()
{
	// Firstly, is our thread still running? If so, ask it to quit or kill it
	if(mythreadID)
	{
		DWORD ret;
		// Request thread to terminate
		TEST_WIN32ERR(!PostThreadMessage(mythreadID, WM_QUIT, 0, 0));
		ret=WaitForSingleObject(mythreadH, 3000);
		TEST_WIN32ERR(ret==WAIT_FAILED);
		if(ret==WAIT_TIMEOUT)
		{
			// Force it then
			TEST_WIN32ERR(!TerminateThread(mythreadH, 1));
		}
		mythreadID=0;
	}

	
	// All done at termination of threadexe() too
	if(we)
	{
		delete we;
		we=NULL;
	}
	if(bmp)
	{
		delete bmp;
		bmp=NULL;
	}
	if(tool)
	{
		tool=NULL;
	}
	if(hwndToolbox)
	{
		DestroyWindow(hwndToolbox);
		hwndToolbox=NULL;
	}
	if(hwndMainMenuPopup)
	{
		DestroyMenu(hwndMainMenuPopup);
		hwndMainMenuPopup=NULL;
	}
	if(hwndMain)
	{
		DestroyWindow(hwndMain);
		hwndMain=NULL;
	}

	// All done in constructor
	if(mythreadH)
	{
		TEST_WIN32ERR(!CloseHandle(mythreadH));
		mythreadH=NULL;
	}
	if(hwndMolCol)
	{
		DestroyWindow(hwndMolCol);
		hwndMolCol=NULL;
	}
	if(hwndMolNo)
	{
		DestroyWindow(hwndMolNo);
		hwndMolNo=NULL;
	}
	if(hacclMain)
	{
		//TEST_WIN32ERR(!DestroyAcceleratorTable(hacclMain));
		hacclMain=NULL;
	}
}

s32 Game::threadexe()
{
    MSG msg; 
	DWORD t1=0;
	s32 ret=0;
	int iter=0;
	char temp[64];

	try {

	// Create the main window. 
	hwndMain = CreateWindowEx(WS_EX_ACCEPTFILES | WS_EX_OVERLAPPEDWINDOW,
		"MainWndClass", name, WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU
		| WS_MINIMIZEBOX, CW_USEDEFAULT, CW_USEDEFAULT,
        100, 100, (HWND) NULL, 
        (HMENU) NULL, hinst, (LPVOID) 0);

    TEST_WIN32ERR(!hwndMain);
	SetWindowLong(hwndMain, 0, (LONG) this);

	// Set up the menus
	hwndMainMenu=GetMenu(hwndMain);
#ifndef SHAREWARE
	TEST_WIN32ERR(!DeleteMenu(hwndMainMenu, 3, MF_BYPOSITION));
	TEST_WIN32ERR(!DrawMenuBar(hwndMain));
#endif
	hwndMainMenuPopup=CreatePopupMenu();
	iter=GetMenuItemCount(hwndMainMenu);
	for(int n=0; n<iter; n++)
	{
		MENUITEMINFO mii;
		char temp[64];
		mii.cbSize=sizeof(mii);
		mii.fMask=MIIM_CHECKMARKS | MIIM_DATA | MIIM_ID | MIIM_STATE |
			MIIM_SUBMENU | MIIM_TYPE;
		mii.dwTypeData=temp;
		mii.cch=64;
		TEST_WIN32ERR(!GetMenuItemInfo(hwndMainMenu, n, TRUE, &mii));
		TEST_WIN32ERR(!InsertMenuItem(hwndMainMenuPopup, n, TRUE, &mii));
	}
	// Put all *.flowgame in quick start
	{
		DWORD MIbase=40000;
		HANDLE ffh;
		WIN32_FIND_DATA fd;
		MENUITEMINFO mii={ sizeof(mii) };
		hwndMainMenuQLoad=CreateMenu();
		TEST_WIN32ERR(hwndMainMenuQLoad==NULL);
		ffh=FindFirstFile("*.flowgame", &fd);
		if(ffh!=INVALID_HANDLE_VALUE)
		{
			do
			{
				TEST_WIN32ERR(!AppendMenu(hwndMainMenuQLoad, MF_STRING, --MIbase, fd.cFileName));
			} while(FindNextFile(ffh, &fd));
			TEST_WIN32ERR(!FindClose(ffh));
		}
		mii.fMask=MIIM_SUBMENU;
		mii.hSubMenu=hwndMainMenuQLoad;
		TEST_WIN32ERR(!SetMenuItemInfo(hwndMainMenu, IDM_FILE_QLOAD, FALSE, &mii));
	}

	// Set up the toolbar
	hwndToolbox=CreateDialogParam(hinst, MAKEINTRESOURCE(IDD_TOOLBOX), hwndMain, (DLGPROC) ToolboxDialogProc, (LPARAM) this);
	TEST_WIN32ERR(!hwndToolbox);
	TEST_WIN32ERR(!CheckDlgButton(hwndToolbox, IDC_TBLINE, BST_CHECKED));
	TEST_WIN32ERR(!SetDlgItemInt(hwndToolbox, IDC_TBPAR1, 3, TRUE));
	alignside=1;
	toolboxdy=0;

	// Create a bitmap for the water to live in
	bmp=new ScreenImage8(hwndMain, 640, 480);
	// Set up the engine & attach bmp to it
	we=new WaterEngine(100, bmp, &prefs);

	// Set up the screen
	{
		HDC dc;
		dc=bmp->getDC();
		SelectObject(dc, GetStockObject(WHITE_PEN));
		MoveToEx(dc, 0, 0, NULL);
		LineTo(dc, bmp->width-1, 0);
		LineTo(dc, bmp->width-1, bmp->height-1);
		LineTo(dc, 0, bmp->height-1);
		LineTo(dc, 0, 0);
		bmp->releaseDC();
	}
	// Alter menus
	updateMenus();

    // Show the window and paint its contents. 
    ShowWindow(hwndMain, nCmdShow); 
	ShowWindow(hwndToolbox, nCmdShow);
    UpdateWindow(hwndMain);
	SetMainWindowSize();
	updateToolbox();

    while( 1 )
    {
		try
		{
        if(paused || PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( !GetMessage( &msg, NULL, 0, 0 ) )
                break;
			if(msg.hwnd==NULL)
			{
				// Msgs from modeless dialogs come through unaddressed
				if(msg.message==WM_COMMAND)
				{
					switch(LOWORD(msg.wParam))
					{
					case IDC_SETMOLNO: // From Set Molecules No dialog
						{
							if(hwndMolNo)
							{
								// Dialog Box closed with new values set
								do
								{
									if(msg.lParam)		// If OK chosen
									{
										automolno=IsDlgButtonChecked(hwndMolNo, IDC_SETNOFROMFPS)==BST_CHECKED;
										if(!automolno)
										{
											// Set mol no
											UINT t;
											BOOL s;
											int type=2;
											t=GetDlgItemInt(hwndMolNo, IDC_NOOFMOLS, &s, FALSE);
											if(!s || t<0 || t>100000)
											{
												MessageBox(hwndMain, "Invalid specified molecules number (must be positive and less than 100,000). Please reenter", "Message from Flow", MB_OK);
												break;
											}
											if(IsDlgButtonChecked(hwndMolNo, IDC_SNM_NORMAL)==BST_CHECKED) type=0;
											if(IsDlgButtonChecked(hwndMolNo, IDC_SNM_LIGHT)==BST_CHECKED)  type=1;
											if(IsDlgButtonChecked(hwndMolNo, IDC_SNM_MIXED)==BST_CHECKED)  type=2;
											we->setNoOfMols(t, type);
											InvalidateRect(hwndMain, NULL, FALSE);
											if(hwndCalibr)
											{
												DestroyWindow(hwndCalibr);
												hwndCalibr=NULL;
											}

										}
										else
										{
											// Reset system
											for(int n=0; n<1000; n++)
												fpsAtMols[n]=0;
											we->setNoOfMols(100, 2);
											fpsidx=-1;
										}
									}
									DestroyWindow(hwndMolNo);
									hwndMolNo=NULL;
								} while(0);
							}
							break;
						}
					case IDC_PREFS_MOLCOLS: // From Mol Colour preferences dialog
						{
							if(hwndMolCol)
							{
								switch(msg.lParam)
								{
								case 0: // Cancelled
									{
										DestroyWindow(hwndMolCol);
										hwndMolCol=NULL;
										break;
									}
								default: // Something been chosen
									{
										u32 ncol, lcol;
										// bottom 8 bits are normalcol, next 8 are lightcol
										// in 332 rgb format
										ncol=msg.lParam & 0xff;
										lcol=(msg.lParam>>8) & 0xff;
										{
											int r,g,b;
											r=(ncol & 0xe0);
											g=(ncol & 0x1c)<<3;
											b=(ncol & 0x03)<<6;
											// Mirror top bits across rest of byte
											r|=(r>>3) | (r>>6);
											g|=(g>>4) | (g>>6);
											b|=(b>>2) | (b>>4) | (b>>6);
											prefs.normalmolcol=(b<<16)+(g<<8)+r;
										}
										{
											int r,g,b;
											r=(lcol & 0xe0);
											g=(lcol & 0x1c)<<3;
											b=(lcol & 0x03)<<6;
											// Mirror top bits across rest of byte
											r|=(r>>3) | (r>>6);
											g|=(g>>4) | (g>>6);
											b|=(b>>2) | (b>>4) | (b>>6);
											prefs.lightmolcol=(b<<16)+(g<<8)+r;
										}
										prefs.write();
										DestroyWindow(hwndMolCol);
										hwndMolCol=NULL;
										break;
									}
								}
							}
							break;
						}
					case IDM_LOADGAME:
						{
							// Special this: central load file routine, invoked in many places
							HANDLE fileh;
							fileh=CreateFile(name, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
							try
							{
								bool fs;
								TEST_WIN32ERR(fileh==INVALID_HANDLE_VALUE);
								paused=true;
								fs=bmp->fullscreen;
								bmp->setFullScreen(false);
								if((automolno=we->load(fileh))) fpsidx=-1;
								bmp->setFullScreen(fs);
								TEST_WIN32ERR(!CloseHandle(fileh));
								fileh=NULL;
							}
							catch(Error e)
							{
								if(fileh && fileh!=INVALID_HANDLE_VALUE)
								{
									CloseHandle(fileh);
									fileh=NULL;
								}
								//throw;
								e.showAsMsg();
							}
							// Repaint window
							InvalidateRect(hwndMain, NULL, FALSE);
							SetMainWindowSize();
							break;
						} 
					}
				}
			}
			// Modeless dialog box dispatching here
			else if(IsWindow(hwndMolNo) && IsDialogMessage(hwndMolNo, &msg));
			else if(IsWindow(hwndMolCol) && IsDialogMessage(hwndMolCol, &msg));
			else
			{
				// Translate accelerators for toolbox too
				do
				{
					if (IsWindow(hwndToolbox))
					{
						if(TranslateAccelerator(hwndToolbox, hacclMain, &msg)) break;
						if(IsDialogMessage(hwndToolbox, &msg)) break;
					}
					if (TranslateAccelerator(hwndMain, hacclMain, &msg) == 0)
					{
						DispatchMessage(&msg); 
					}
				} while(0);
			}
        }

		if(!paused)
		{
		if(t1==0) t1=GetTickCount();
		else if(GetTickCount()-t1>=1000)
		{
			HDC dc;
			int _fps,fpsAtMolsIdx;
			// Calc fps
			_fps=iter;
			iter=0;
			if(fpsidx==-1)
			{
				fps[1]=fps[2]=fps[3]=fps[4]=_fps;
				fpsidx=0;
			}
			fps[fpsidx++]=_fps;
			if(fpsidx==5) fpsidx=0;
			_fps=(fps[0]+fps[1]+fps[2]+fps[3]+fps[4])/5;
			fpsAtMolsIdx=we->mols/100;
			fpsAtMols[fpsAtMolsIdx]=_fps;
			if(automolno)
			{
				if(!hwndCalibr)
				{
					// Disable window and force topmost
					/*SetWindowPos(hwndMain, HWND_TOPMOST, 0, 0, 0, 0,
						SWP_NOMOVE | SWP_NOSIZE);
					/EnableWindow(hwndMain, FALSE);*/
					hwndCalibr=CreateDialog(hinst, MAKEINTRESOURCE(IDD_CALIBRATING), hwndMain, (DLGPROC) CalibrDialogProc);
					SetForegroundWindow(hwndMain);
				}
				// Make mol no 95% of the highest frame rate
				if(_fps>150)
				{
					we->setNoOfMols(u32(we->mols+200), 2);
				}
				else if(_fps<int(fpsAtMols[1]*0.95))
				{
					we->setNoOfMols(u32(we->mols-100), 2);
					automolno=false;
					SetWindowPos(hwndMain, HWND_NOTOPMOST, 0, 0, 0, 0,
						SWP_NOMOVE | SWP_NOSIZE);
					EnableWindow(hwndMain, TRUE);
					DestroyWindow(hwndCalibr);
					hwndCalibr=NULL;
				}
				else
				{
					we->setNoOfMols(u32(we->mols+100), 2);
				}
			}
#ifdef SHAREWARE
			{
				DWORD t=10*60-(GetTickCount()-timeofcreation)/1000;
				t--;
				sprintf(temp, "%02d:%02d", t/60, t % 60);
				// Write to bitmap
				dc=bmp->getDC();
				SelectObject(dc, GetStockObject(SYSTEM_FONT));
				TextOut(dc, bmp->width-50, 1, temp, strlen(temp));
				bmp->releaseDC();
			}
#endif
			if(debug)
			{
				sprintf(temp, "%d, FPS: %d  ", we->mols, _fps);
				// Write to bitmap
				dc=bmp->getDC();
				SelectObject(dc, GetStockObject(SYSTEM_FONT));
				TextOut(dc, bmp->width-100, 10, temp, strlen(temp));
				bmp->releaseDC();
			}
			t1=GetTickCount();
		}

        {
			HDC dc;
			POINT p={0, 0};
#ifdef SHAREWARE
			if(GetTickCount()-timeofcreation>10*60*1000)
			{
				paused=true;
				MessageBox(hwndMain, "You may only play the game for ten minutes on the UNREGISTERED version.\nPlease register the game to play for as long as you like!", "Message from Flow", MB_OK);
			}
			else
#endif
			{
				bmp->getAddr();
				we->update();
				bmp->releaseAddr();
			}
			ClientToScreen(hwndMain, &p);
			dc=GetDC(hwndMain);
			bmp->paint(dc, p.x, p.y, bmpscale);
			iter++;
			ReleaseDC(hwndMain, dc);
			//Sleep(1);
        }
		}
		}
		catch(Error e)
		{
			// Provide friendly error handling
			throw;
		}
    }
	}
	catch(Error e)
	{
		char temp[256];
		e.toString(temp, 256);
		MessageBox(hwndMain, temp, "Error:", MB_SYSTEMMODAL | MB_OK);
		ret=1;
	}
	catch(char *e)
	{
		MessageBox(hwndMain, e, "FATAL ERROR:", MB_SYSTEMMODAL | MB_OK);
		ret=1;
	}
	if(!ret) ret=msg.wParam;

	// Tidy up after ourselves (done again in destructor just in case thread hung)
	if(we)
	{
		delete we;
		we=NULL;
	}
	if(bmp)
	{
		delete bmp;
		bmp=NULL;
	}
	if(tool)
	{
		tool=NULL;
	}
	if(hwndToolbox)
	{
		DestroyWindow(hwndToolbox);
		hwndToolbox=NULL;
	}
	if(hwndMainMenuPopup)
	{
		DestroyMenu(hwndMainMenuPopup);
		hwndMainMenuPopup=NULL;
	}
	if(hwndMain)
	{
		DestroyWindow(hwndMain);
		hwndMain=NULL;
	}

	// Request parent thread to delete us
	mythreadID=0;									// we'll be dead soon, so prevent an error
	TEST_WIN32ERR(!PostThreadMessage(parentthreadID, WM_CLOSEGAMEREQ, 0, (LPARAM) this));
	return ret;
} 

LRESULT Game::wndproc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	struct Decision {
		bool repaint;
		bool boundbox;
		bool unboundbox;
	};
	Decision *d;
	int decstorage=0;
	d=(Decision *) &decstorage;

	if(hwnd==hwndToolbox) return DefWindowProc(hwnd, uMsg, wParam, lParam);

	switch(uMsg)
	{
	case WM_DESTROY:
		{
			PostQuitMessage(0);
			/*if(hwnd==hwndMain)
				SendMessage(hwndMain, WM_QUIT, 0, 0);*/
			return 0;
		}
    case WM_PALETTECHANGED:
        if ((HWND)wParam == hwndMain)
            break;
        // fall through to WM_QUERYNEWPALETTE
	case WM_SYSCOLORCHANGE:
		{
			// install our palette here
			if(bmp)
			{
				bmp->installPalette();
				we->palettechanged();
				return TRUE;
			}
			else
				return FALSE;
		}
	case WM_DISPLAYCHANGE:
		{
			if(bmp)
			{
				// Odd that it needs this
				Sleep(3000);
				bmp->modeChanged();
				we->palettechanged();
				SetMainWindowSize();
			}
			break;
		}
	case WM_MOVE:
		{
			if(hwndToolbox)
			{
				// Align toolbox to main game window
				RECT r;
				WINDOWPOS wp;
				GetWindowRect(hwndToolbox, &r);
				wp.hwnd=hwndToolbox;
				wp.hwndInsertAfter=hwndMain;
				wp.x=r.left;
				wp.y=r.top;
				wp.cx=r.right-r.left;
				wp.cy=r.bottom-r.top;
				wp.flags=SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE;
				alignToolbox(&wp, false);
				TEST_WIN32ERR(!SetWindowPos(wp.hwnd, wp.hwndInsertAfter, wp.x, wp.y, wp.cx, wp.cy, wp.flags));
			}
			break;
		}
	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC dc;
			POINT p={0, 0};
			ClientToScreen(hwndMain, &p);
			dc=BeginPaint(hwnd, &ps);
			bmp->paint(dc, p.x, p.y, bmpscale);
			EndPaint(hwnd, &ps);
			return 0;
		}
	case WM_INITMENU:
		{
			updateMenus();
			return 0;
		}
	case WM_KEYUP:
    case WM_KEYDOWN:
		{
			bool keydown=(uMsg==WM_KEYDOWN);
			// No repeats
			if(keydown && lParam & (1<<30)) break;
			switch((int) wParam )
			{
			case VK_ESCAPE:
				{
					if(keydown) bmp->setFullScreen(0);
					return 0;
				}
			case VK_END:
				{
					if(keydown) debug=!debug;
					return 0;
				}
			default:
				{
					UINT ch;
					ch=MapVirtualKey(wParam, 2);
					switch(ch)
					{
					case 'X':		// horiz line draw
					case 'x':
						{
							lockY=keydown;
							break;
						}
					case 'V':		// vert line draw
					case 'v':
						{
							lockX=keydown;
							break;
						}
#ifdef _DEBUG
					case 'G':
					case 'g':
						{
							if(keydown)
							{
								u32 col;
								u8 *addr;
								char temp[64];
								addr=(u8 *) bmp->getAddr();
								col=addr[0];
								bmp->releaseAddr();
								sprintf(temp, "Col at 0,0 is %d", col);
								MessageBox(NULL, temp, "Msg", MB_OK);
							}
							break;
						}
#endif
					}
					break;
				}
			}
			break;
		}
	case WM_DROPFILES:
		{
			HDROP hDrop=(HDROP) wParam;
			DragQueryFile(hDrop, 0, name, sizeof(name));
			// Invoke load
			PostMessage(NULL, WM_COMMAND, IDM_LOADGAME, 0);
			DragFinish(hDrop);
			return 0;
		}
	case WM_COMMAND:
		{
			switch(LOWORD(wParam))
			{
			case IDM_NEWGAME:
				{
					// Can't new game in full screen mode
					if(bmp->fullscreen)
					{
						bmp->setFullScreen(false);
						SetMainWindowSize();
					}
					TEST_WIN32ERR(!PostThreadMessage(parentthreadID, WM_NEWGAMEREQ, (WPARAM) SW_SHOWNORMAL, 0));
					break;
				}
			case IDM_SAVEGAME:
				{
#ifdef SHAREWARE
					MessageBox(hwndMain, "This is disabled in the UNREGISTERED version of this game. Please register (see Help for details) to be able to save games",
						"Message from Flow", MB_OK);
#else
					// Open Save As dialog box
					OPENFILENAME ofn;
					char exepath[_MAX_PATH];
					GetCurrentDirectory(_MAX_PATH, exepath);
					ZeroMemory(&ofn, sizeof(ofn));
					ofn.lStructSize=sizeof(ofn);
					ofn.hwndOwner=hwndMain;
					ofn.lpstrFilter=(LPCTSTR) "Flow Game files\0*.flowgame\0All files\0*.*\0\0\0";
					ofn.lpstrCustomFilter=NULL;
					ofn.nFilterIndex=1;
					ofn.lpstrFile=name;
					ofn.nMaxFile=sizeof(name);
					ofn.lpstrFileTitle=NULL;
					ofn.lpstrInitialDir=exepath;
					ofn.lpstrTitle="Save Flow Game As ...";
					ofn.Flags=OFN_EXPLORER | OFN_LONGNAMES
						| OFN_PATHMUSTEXIST;
					ofn.lpstrDefExt="flowgame";
					if(GetSaveFileName(&ofn))
					{
						HANDLE fileh;
						strcpy(ofn.lpstrFile+ofn.nFileExtension-1, ".flowgame");
						fileh=CreateFile(name, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_NEW,
							FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
						if(fileh==INVALID_HANDLE_VALUE)
						{
							if(MessageBox(hwndMain, "This file already exists. Overwrite?",
								"Question from Flow", MB_YESNO)!=IDYES)
								break;
							fileh=CreateFile(name, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS,
								FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL);
							TEST_WIN32MSG(fileh==INVALID_HANDLE_VALUE);
						}
						try
						{
							we->save(fileh, automolno);
							TEST_WIN32ERR(!CloseHandle(fileh));
							fileh=NULL;
						}
						catch(Error e)
						{
							if(fileh)
							{
								CloseHandle(fileh);
								fileh=NULL;
							}
							e.showAsMsg();
						}
					}
#endif
					break;
				}
			case IDM_LOADGAME:
				{
					// Open Open File dialog box
					OPENFILENAME ofn;
					char exepath[_MAX_PATH];
					GetCurrentDirectory(_MAX_PATH, exepath);
					ZeroMemory(&ofn, sizeof(ofn));
					ofn.lStructSize=sizeof(ofn);
					ofn.hwndOwner=hwndMain;
					ofn.lpstrFilter=(LPCTSTR) "Flow Game files\0*.flowgame\0All files\0*.*\0\0\0";
					ofn.lpstrCustomFilter=NULL;
					ofn.nFilterIndex=1;
					ofn.lpstrFile=name;
					ofn.nMaxFile=sizeof(name);
					ofn.lpstrFileTitle=NULL;
					ofn.lpstrInitialDir=exepath;
					ofn.lpstrTitle="Open Flow Game File ...";
					ofn.Flags=OFN_EXPLORER | OFN_LONGNAMES | OFN_HIDEREADONLY
						| OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
					ofn.lpstrDefExt="flowgame";
					if(GetOpenFileName(&ofn))
					{
						// Invoke loading
						strcpy(ofn.lpstrFile+ofn.nFileExtension-1, ".flowgame");
						PostMessage(NULL, WM_COMMAND, IDM_LOADGAME, 0);
					}
					else
					{
						DWORD errcode;
						errcode=CommDlgExtendedError();
						if(errcode)
						{
							DECLARE_ERROR(e, "The Load Dialog box wouldn't open");
							throw e;
						}
					}
					break;
				}
			case IDM_CLOSEGAME:
				{
					PostQuitMessage(0);
					break;
				}
			case IDM_QUIT:
				{
					TEST_WIN32ERR(!PostThreadMessage(parentthreadID, WM_QUIT, 0, 0));
					break;
				}
			case IDM_PAUSEGAME:
				{
					paused=!paused;
					break;
				}
			case IDM_DOUBLESIZE:
				{
					bmpscale=3-bmpscale;
					SetMainWindowSize();
					break;
				}
			case IDM_FULLSCREEN:
				{
					bmp->setFullScreen(!bmp->fullscreen);
					SetMainWindowSize();
					break;
				}
			case IDM_WEIGHWATER:
				{
					we->densities=!we->densities;
					updateToolbox();
					break;
				}
			case IDM_FILTERON:
				{
					we->filter=!we->filter;
					updateToolbox();
					break;
				}
			case IDM_GO800X600:
				{
					we->setSize(800, 600);
					SetMainWindowSize();
					break;
				}
			case IDM_GO1024X768:
				{
					we->setSize(1024, 768);
					SetMainWindowSize();
					break;
				}
			case IDM_GO1280X1024:
				{
					we->setSize(1280, 1024);
					SetMainWindowSize();
					break;
				}
			case IDM_GO1600X1200:
				{
					we->setSize(1600, 1200);
					SetMainWindowSize();
					break;
				}
			case IDM_CHANGEMOLNO:
				{
					if(!IsWindow(hwndMolNo))
					{
						hwndMolNo=CreateDialogParam(hinst, MAKEINTRESOURCE(IDD_MOLNOBOX), hwndMain, (DLGPROC) MolNoDialogProc, (LPARAM) this);
						TEST_WIN32ERR(hwndMolNo==NULL);
						ShowWindow(hwndMolNo, SW_SHOWNORMAL);
					}
					break;
				}
			case IDM_PREFS_MOLCOLS:
				{
					if(!IsWindow(hwndMolCol))
					{
						hwndMolCol=CreateDialogParam(hinst, MAKEINTRESOURCE(IDD_PREFS_MOLCOLS), hwndMain, (DLGPROC) PrefsMolColDialogProc, (LPARAM) this);
						TEST_WIN32ERR(hwndMolCol==NULL);
						ShowWindow(hwndMolCol, SW_SHOWNORMAL);
					}
					break;
				}
			case IDM_PREFS_USEGDIBLITTINGONLY:
				{
					Bitmap::GDIOnly=!Bitmap::GDIOnly;
					d->repaint=true;
					break;
				}
			case IDM_BUY_PURCHASEINFORMATION:
				{
					char temp[1024];
					LPTSTR a;
					SHELLEXECUTEINFO sei;
					GetFullPathName("Docs\\purchase.html", 1024, temp, &a);
					memset(&sei, 0, sizeof(sei));
					sei.cbSize=sizeof(sei);
					sei.fMask=0;
					sei.hwnd=hwndMain;
					sei.lpFile=temp;
					ShellExecuteEx(&sei);
					break;
				}
			case IDM_ABOUTWATER:
				{
					int ret;
					ret=DialogBox(hinst, MAKEINTRESOURCE(IDD_ABOUTBOX), hwndMain, (DLGPROC) InfoDialogProc);
					if(ret==IDC_GOFLOWWWW)
					{
						SHELLEXECUTEINFO sei;
						memset(&sei, 0, sizeof(sei));
						sei.cbSize=sizeof(sei);
						sei.fMask=0;
						sei.hwnd=hwndMain;
						sei.lpFile="http://www.nedprod.com/programs/Win32/Flow/";
						TEST_WIN32MSG(!ShellExecuteEx(&sei));
					}
					if(ret!=IDD_GOTOHELP)
						break;
					// Let fall through
				}
			case IDM_HELPCONTENTS:
				{
					char temp[1024];
					LPTSTR a;
					SHELLEXECUTEINFO sei;
					GetFullPathName("Docs\\index.html", 1024, temp, &a);
					memset(&sei, 0, sizeof(sei));
					sei.cbSize=sizeof(sei);
					sei.fMask=0;
					sei.hwnd=hwndMain;
					sei.lpFile=temp;
					ShellExecuteEx(&sei);
					break;
				}
			default:
				{
					// See if it's one of the quick loads
					if(LOWORD(wParam)<40000)
					{
						GetMenuString(hwndMainMenuQLoad, LOWORD(wParam), name, sizeof(name), 0);
						// Invoke loading
						PostMessage(NULL, WM_COMMAND, IDM_LOADGAME, 0);
					}
					else
						return DefWindowProc(hwnd, uMsg, wParam, lParam);
				}
			}
			break;
		}
	case WM_MBUTTONDOWN:
		{
			// Open window menu as context
			POINT p;
			p.x=LOWORD(lParam);
			p.y=HIWORD(lParam);
			ClientToScreen(hwndMain, &p);
			TrackPopupMenu(hwndMainMenuPopup, TPM_CENTERALIGN | TPM_TOPALIGN,
				p.x, p.y, 0, hwndMain, NULL);
			break;
		}

	case WM_LBUTTONDOWN:
	case WM_RBUTTONDOWN:
		{
			drawing=TRUE;
			if(!lockX) oldbmpX=LOWORD(lParam)/bmpscale;
			if(!lockY) oldbmpY=HIWORD(lParam)/bmpscale;
			if(tool)
			{
				int keptpar1;
				tool->x1=tool->x2=oldbmpX;
				tool->y1=tool->y2=oldbmpY;
				tool->mods=TOOL_BUILDMODS(wParam & MK_LBUTTON, wParam & MK_RBUTTON, wParam & MK_SHIFT);
				tool->par1=keptpar1=GetDlgItemInt(hwndToolbox, IDC_TBPAR1, NULL, TRUE);
				tool->lockX=lockX;
				tool->lockY=lockY;
				tool->dc=bmp->getDC();
				tool->begin();
				bmp->releaseDC();
				lockX=tool->lockX;
				lockY=tool->lockY;
				oldbmpX=tool->x2;
				oldbmpY=tool->y2;
				if(tool->par1!=keptpar1)
				{
					TEST_WIN32ERR(!SetDlgItemInt(hwndToolbox, IDC_TBPAR1, tool->par1, TRUE));
				}
			}
			if(paused) d->repaint=true;
			d->boundbox=true;
			break;
		}
	case WM_LBUTTONUP:
	case WM_RBUTTONUP:
		{
			drawing=FALSE;
			ClipCursor(NULL);
			if(tool)
			{
				tool->x2=oldbmpX;
				tool->y2=oldbmpY;
				tool->mods=TOOL_BUILDMODS(wParam & MK_LBUTTON, wParam & MK_RBUTTON, wParam & MK_SHIFT);
				tool->par1=GetDlgItemInt(hwndToolbox, IDC_TBPAR1, NULL, TRUE);
				tool->lockX=lockX;
				tool->lockY=lockY;
				tool->dc=bmp->getDC();
				tool->end();
				bmp->releaseDC();
				lockX=tool->lockX;
				lockY=tool->lockY;
				oldbmpX=tool->x2;
				oldbmpY=tool->y2;
			}
			break;
		}
	case WM_MOUSEWHEEL:
		{
			break;
		}
	case WM_MOUSEMOVE:
		{
			if(drawing)
			{
				if(tool)
				{
					tool->x1=oldbmpX;
					tool->y1=oldbmpY;
				}
				if(!lockX) oldbmpX=LOWORD(lParam)/bmpscale;
				if(!lockY) oldbmpY=HIWORD(lParam)/bmpscale;
				if(tool)
				{
					int keptpar1;
					tool->x2=oldbmpX;
					tool->y2=oldbmpY;
					tool->mods=TOOL_BUILDMODS(wParam & MK_LBUTTON, wParam & MK_RBUTTON, wParam & MK_SHIFT);
					tool->par1=keptpar1=GetDlgItemInt(hwndToolbox, IDC_TBPAR1, NULL, TRUE);
					tool->lockX=lockX;
					tool->lockY=lockY;
					tool->dc=bmp->getDC();
					tool->poll();
					bmp->releaseDC();
					lockX=tool->lockX;
					lockY=tool->lockY;
					oldbmpX=tool->x2;
					oldbmpY=tool->y2;
					if(tool->par1!=keptpar1)
					{
						TEST_WIN32ERR(!SetDlgItemInt(hwndToolbox, IDC_TBPAR1, tool->par1, TRUE));
					}
				}
				if(paused) d->repaint=true;
			}
			else
			{
				if(!lockX) oldbmpX=LOWORD(lParam)/bmpscale;
				if(!lockY) oldbmpY=HIWORD(lParam)/bmpscale;
			}
			break;
		}

	default:
		{
			return DefWindowProc(hwnd, uMsg, wParam, lParam);
		}
	}

	// Implement decisions taken above
	if(d->boundbox)
	{
		// Confine the cursor to our client area
		RECT r;
		POINT p;
		p.x=0; p.y=0;
		ClientToScreen(hwndMain, &p);
		r.left=p.x; r.top=p.y;
		p.x=bmp->width*bmpscale; p.y=bmp->height*bmpscale;
		ClientToScreen(hwndMain, &p);
		r.right=p.x; r.bottom=p.y;
		ClipCursor(&r);
	}
	if(d->repaint)
	{
		HDC dc;
		POINT p={0, 0};
		bmp->getAddr();
		we->render(true);
		bmp->releaseAddr();
		ClientToScreen(hwndMain, &p);
		dc=GetDC(hwndMain);
		bmp->paint(dc, p.x, p.y, bmpscale);
		ReleaseDC(hwndMain, dc);
	}

	return 0;
}

void Game::updateMenus()
{
	// Checked/unchecked things
	{
		MENUITEMINFO mii;
		mii.cbSize=sizeof(MENUITEMINFO);
		mii.fMask=MIIM_STATE;
		mii.fState=(paused) ? MFS_CHECKED : MFS_UNCHECKED;
		TEST_WIN32ERR(SetMenuItemInfo(hwndMainMenu, IDM_PAUSEGAME, FALSE, &mii)==0);
		mii.fMask=MIIM_STATE;
		mii.fState=(bmpscale==2) ? MFS_CHECKED : MFS_UNCHECKED;
		TEST_WIN32ERR(SetMenuItemInfo(hwndMainMenu, IDM_DOUBLESIZE, FALSE, &mii)==0);
		mii.fMask=MIIM_STATE;
		mii.fState=(we->densities) ? MFS_CHECKED : MFS_UNCHECKED;
		TEST_WIN32ERR(SetMenuItemInfo(hwndMainMenu, IDM_WEIGHWATER, FALSE, &mii)==0);
		mii.fMask=MIIM_STATE;
		mii.fState=(!we->filter) ? MFS_CHECKED : MFS_UNCHECKED;
		TEST_WIN32ERR(SetMenuItemInfo(hwndMainMenu, IDM_FILTERON, FALSE, &mii)==0);
		mii.fMask=MIIM_STATE;
		mii.fState=(Bitmap::GDIOnly) ? MFS_CHECKED : MFS_UNCHECKED;
		TEST_WIN32ERR(SetMenuItemInfo(hwndMainMenu, IDM_PREFS_USEGDIBLITTINGONLY, FALSE, &mii)==0);
	}
}

void Game::SetMainWindowSize()
{
	int oldalignside=alignside;
	int oldtoolboxdy=toolboxdy;
	RECT r;
	r.left=0;
	r.top=0;
	r.right=bmp->width*bmpscale;
	r.bottom=bmp->height*bmpscale;
	AdjustWindowRectEx(&r, GetWindowLong(hwndMain, GWL_STYLE),
		GetMenu(hwndMain) != NULL, GetWindowLong(hwndMain, GWL_EXSTYLE));
    SetWindowPos(hwndMain, NULL, 0, 0, r.right-r.left, r.bottom-r.top,
        SWP_NOZORDER|SWP_NOMOVE|SWP_NOACTIVATE);
	{
		char temp[_MAX_PATH+12]="Flow - ";
		strcat(temp, name);
		strcat(temp, " *");
		SetWindowText(hwndMain, temp);
	}
	// Attach toolbox in case it got lost
	if(oldalignside==0) alignside=1; else alignside=oldalignside;
	toolboxdy=oldtoolboxdy;
	PostMessage(hwndMain, WM_MOVE, 0, 0);
}

void Game::alignToolbox(LPWINDOWPOS wp, bool update)
{
	RECT r;
	POINT p;
	TEST_WIN32ERR(!GetClientRect(hwndMain, &r));
	p.x=r.left;		p.y=r.top;
	TEST_WIN32ERR(!ClientToScreen(hwndMain, &p));
	r.left=p.x;		r.top=p.y;
	p.x=r.right;	p.y=r.bottom;
	TEST_WIN32ERR(!ClientToScreen(hwndMain, &p));
	r.right=p.x;		r.bottom=p.y;
	
	if(update)
	{
		// Decide if user wants alignment
		if(wp->y>r.top-50 && wp->y<r.bottom)
		{
			int hotspotx1=r.left-wp->cx;
			if(wp->x>hotspotx1-25 && wp->x<hotspotx1+25)
				alignside=1;
			else if(wp->x>r.right-25 && wp->x<r.right+25)
				alignside=2;
			else
				alignside=0;
		}
	}

	if(alignside==1)		// Align to left
	{
		wp->x=r.left-wp->cx;
		if(wp->x<0) wp->x=0;
		if(!update)
		{
			wp->y=r.top+toolboxdy;
		}
		if(wp->y<r.top) wp->y=r.top;
		if(wp->y>r.bottom-wp->cy) wp->y=r.bottom-wp->cy;
		if(update)
		{
			toolboxdy=wp->y-r.top;
		}
	}
	else if(alignside==2)	// Align to right
	{
		int t;
		wp->x=r.right;
		t=GetSystemMetrics(SM_CXFULLSCREEN);
		if(wp->x>t-wp->cx) wp->x=t-wp->cx;
		if(!update)
		{
			wp->y=r.top+toolboxdy;
		}
		if(wp->y<r.top) wp->y=r.top;
		if(wp->y>r.bottom-wp->cy) wp->y=r.bottom-wp->cy;
		if(update)
		{
			toolboxdy=wp->y-r.top;
		}
	}
}

void Game::updateToolbox()
{
	// Update parameter value based on newly chosen tool
	if     (IsDlgButtonChecked(hwndToolbox, IDC_TBLINE     )==BST_CHECKED) tool=&linetool;
	else if(IsDlgButtonChecked(hwndToolbox, IDC_TBCAPILLARY)==BST_CHECKED) tool=&captool;
	else if(IsDlgButtonChecked(hwndToolbox, IDC_TBFILTER   )==BST_CHECKED) tool=&filtAtool;
	else if(IsDlgButtonChecked(hwndToolbox, IDC_TBLFILTER  )==BST_CHECKED) tool=&filtRtool;
	else if(IsDlgButtonChecked(hwndToolbox, IDC_TBNFILTER  )==BST_CHECKED) tool=&filtBtool;
	else if(IsDlgButtonChecked(hwndToolbox, IDC_TBUPUMP    )==BST_CHECKED) tool=&pumpUptool;
	else if(IsDlgButtonChecked(hwndToolbox, IDC_TBLPUMP    )==BST_CHECKED) tool=&pumpLefttool;
	else if(IsDlgButtonChecked(hwndToolbox, IDC_TBRPUMP    )==BST_CHECKED) tool=&pumpRighttool;
	if(tool)
	{
		EnableWindow(GetDlgItem(hwndToolbox, IDC_TBPAR1), TRUE);
		TEST_WIN32ERR(!SetDlgItemInt(hwndToolbox, IDC_TBPAR1, tool->par1, TRUE));
		tool->we=we;
	}
	else
	{
		EnableWindow(GetDlgItem(hwndToolbox, IDC_TBPAR1), FALSE);
	}
	// Update indicators
	TEST_WIN32ERR(!CheckDlgButton(hwndToolbox, IDC_TBPAUSED, (paused) ? BST_CHECKED : BST_UNCHECKED));
	TEST_WIN32ERR(!CheckDlgButton(hwndToolbox, IDC_TBLIQDENSON, (we->densities) ? BST_CHECKED : BST_UNCHECKED));
	TEST_WIN32ERR(!CheckDlgButton(hwndToolbox, IDC_TBFILTERON, (!we->filter) ? BST_CHECKED : BST_UNCHECKED));
}