/* Tools.cpp
Implements the Tools classes
(C) 2000 N. Douglas
15th May 2000
*/

#include "Tool.h"
#include "WaterEngine.h"
#include <assert.h>

ToolLine::ToolLine()
{
	par1=3;
}

ToolAFilter::ToolAFilter()
{
	par1=3;
}
u32 ToolAFilter::getcol() const
{
	return 0x00c0c0c0;
	//return we->filtAscrcol;
}

ToolRFilter::ToolRFilter()
{
	par1=3;
}
u32 ToolRFilter::getcol() const
{
	return 0x00c0c0f0;
	//return we->filtRscrcol;
}

ToolBFilter::ToolBFilter()
{
	par1=3;
}
u32 ToolBFilter::getcol() const
{
	return 0x00f0c0c0;
	//return we->filtBscrcol;
}

ToolLine::~ToolLine()
{
	end();
}

void ToolLine::begin()
{
	int penwidth;
	COLORREF pencol;
	if(par1<1) par1=1;
	if(TOOL_ISSHIFTED(mods)) penwidth=1; else penwidth=par1;
	if(TOOL_ISRBUTTON(mods))
	{
		if(penwidth>1) penwidth++;
		pencol=0;
	}
	else
	{
		pencol=getcol();
	}
	drawingpen=CreatePen(PS_SOLID, penwidth, pencol);
	TEST_WIN32ERR(drawingpen==NULL);
	x2=x1+1;									// Ensure it draws at least one point
	y2=y1;
	poll();
	x2=x1;
}

void ToolLine::poll()
{
	if(drawingpen)
	{
		SelectObject(dc, drawingpen);
	}
	TEST_WIN32ERR(!MoveToEx(dc, x1, y1, NULL));
	TEST_WIN32ERR(!LineTo(dc, x2, y2));
}

void ToolLine::end()
{
	if(drawingpen)
	{
		TEST_WIN32ERR(!DeleteObject(drawingpen));
		drawingpen=NULL;
	}
}





ToolCap::ToolCap()
{
	par1=10;
}

ToolCap::~ToolCap()
{
	end();
}

void ToolCap::begin()
{
	if(TOOL_ISRBUTTON(mods))
		drawingpen=CreatePen(PS_SOLID, 1, 0x00000000);
	else
		drawingpen=CreatePen(PS_SOLID, 1, 0x00ffffff);
	TEST_WIN32ERR(drawingpen==NULL);
	lockX=true;
	if(par1<1) par1=1;
	if(par1>4096) par1=4096;
	x2=x1;
	y2=y1+1;
	poll();
}

void ToolCap::poll()
{
	int y;
	if(drawingpen)
	{
		SelectObject(dc, drawingpen);
	}
	x1=x1 & ~1;
	y=y1;
	while(y!=y2)
	{
		for(int x=0; x<(par1*2); x+=2)
		{
			TEST_WIN32ERR(!MoveToEx(dc, x1+x, y, NULL));
			TEST_WIN32ERR(!LineTo(dc, x1+x+1, y));
		}
		if(y2>y) y++;
		else y--;
	}
}

void ToolCap::end()
{
	if(drawingpen)
	{
		TEST_WIN32ERR(!DeleteObject(drawingpen));
		drawingpen=NULL;
	}
	lockX=false;
}





ToolPumpUp::ToolPumpUp()
{
	par1=10;
}

ToolPumpUp::~ToolPumpUp()
{
	end();
}

void ToolPumpUp::begin()
{
	if(TOOL_ISRBUTTON(mods))
	{
		drawingpenO=CreatePen(PS_SOLID, 1, 0x00000000);
		drawingpenF=CreatePen(PS_SOLID, 1, 0x00000000);
	}
	else
	{
		drawingpenO=CreatePen(PS_SOLID, 1, 0x00ffffff);
		drawingpenF=CreatePen(PS_SOLID, 1, 0x00808000);
	}
	TEST_WIN32ERR(drawingpenO==NULL || drawingpenF==NULL);
	lockX=true;
	if(par1<1) par1=1;
	if(par1>4096) par1=4096;
	x2=x1;
	y2=y1+1;
	poll();
}

void ToolPumpUp::poll()
{
	int y;
	x1=x1 & ~1;
	y=y1;
	while(y!=y2)
	{
		SelectObject(dc, drawingpenO);
		TEST_WIN32ERR(!MoveToEx(dc, x1, y, NULL));
		TEST_WIN32ERR(!LineTo(dc, x1+par1, y));
		SelectObject(dc, drawingpenF);
		TEST_WIN32ERR(!MoveToEx(dc, x1+1, y, NULL));
		TEST_WIN32ERR(!LineTo(dc, x1+par1-1, y));
		if(y2>y) y++;
		else y--;
	}
}

void ToolPumpUp::end()
{
	if(drawingpenO)
	{
		TEST_WIN32ERR(!DeleteObject(drawingpenO));
		drawingpenO=NULL;
	}
	if(drawingpenF)
	{
		TEST_WIN32ERR(!DeleteObject(drawingpenF));
		drawingpenF=NULL;
	}
	lockX=false;
}





ToolPumpLeft::ToolPumpLeft()
{
	par1=10;
}

ToolPumpLeft::~ToolPumpLeft()
{
	end();
}

void ToolPumpLeft::begin()
{
	if(TOOL_ISRBUTTON(mods))
	{
		drawingpenO=CreatePen(PS_SOLID, 1, 0x00000000);
		drawingpenF=CreatePen(PS_SOLID, 1, 0x00000000);
	}
	else
	{
		drawingpenO=CreatePen(PS_SOLID, 1, 0x00ffffff);
		drawingpenF=CreatePen(PS_SOLID, 1, 0x00008000);
	}
	TEST_WIN32ERR(drawingpenO==NULL || drawingpenF==NULL);
	lockY=true;
	if(par1<1) par1=1;
	if(par1>4096) par1=4096;
	x2=x1+1;
	y2=y1;
	poll();
}

void ToolPumpLeft::poll()
{
	int x;
	x=x1;
	while(x!=x2)
	{
		SelectObject(dc, drawingpenO);
		TEST_WIN32ERR(!MoveToEx(dc, x, y1, NULL));
		TEST_WIN32ERR(!LineTo(dc, x, y1-par1));
		SelectObject(dc, drawingpenF);
		TEST_WIN32ERR(!MoveToEx(dc, x, y1-1, NULL));
		TEST_WIN32ERR(!LineTo(dc, x, y1-par1+1));
		if(x2>x) x++;
		else x--;
	}
}

void ToolPumpLeft::end()
{
	if(drawingpenO)
	{
		TEST_WIN32ERR(!DeleteObject(drawingpenO));
		drawingpenO=NULL;
	}
	if(drawingpenF)
	{
		TEST_WIN32ERR(!DeleteObject(drawingpenF));
		drawingpenF=NULL;
	}
	lockY=false;
}






ToolPumpRight::ToolPumpRight()
{
	par1=10;
}

ToolPumpRight::~ToolPumpRight()
{
	end();
}

void ToolPumpRight::begin()
{
	if(TOOL_ISRBUTTON(mods))
	{
		drawingpenO=CreatePen(PS_SOLID, 1, 0x00000000);
		drawingpenF=CreatePen(PS_SOLID, 1, 0x00000000);
	}
	else
	{
		drawingpenO=CreatePen(PS_SOLID, 1, 0x00ffffff);
		drawingpenF=CreatePen(PS_SOLID, 1, 0x00000080);
	}
	TEST_WIN32ERR(drawingpenO==NULL || drawingpenF==NULL);
	lockY=true;
	if(par1<1) par1=1;
	if(par1>4096) par1=4096;
	x2=x1+1;
	y2=y1;
	poll();
}

void ToolPumpRight::poll()
{
	int x;
	x=x1;
	while(x!=x2)
	{
		SelectObject(dc, drawingpenO);
		TEST_WIN32ERR(!MoveToEx(dc, x, y1, NULL));
		TEST_WIN32ERR(!LineTo(dc, x, y1-par1));
		SelectObject(dc, drawingpenF);
		TEST_WIN32ERR(!MoveToEx(dc, x, y1-1, NULL));
		TEST_WIN32ERR(!LineTo(dc, x, y1-par1+1));
		if(x2>x) x++;
		else x--;
	}
}

void ToolPumpRight::end()
{
	if(drawingpenO)
	{
		TEST_WIN32ERR(!DeleteObject(drawingpenO));
		drawingpenO=NULL;
	}
	if(drawingpenF)
	{
		TEST_WIN32ERR(!DeleteObject(drawingpenF));
		drawingpenF=NULL;
	}
	lockY=false;
}

