/* WaterEngine.h
Defines the Water Engine
(C) 2000 N. Douglas
12th April 2000
*/

#include "common.h"
#include "win32bitmap.h"

// inline the automata rules?
#define ENGOPTS
// different calling protocol?
#define ENGOPTS2 __inline

typedef struct Molecule_t
{
	int light:1;					// if it's a "light" molecule
	u32 pixeloffset;				// Offset of pixels into the screen image
	u32 pixelcolour;				// Colour of pixel in RGB
	u32 scrcol;						// Colour in physical form
	u32 oldscrcol;					// Colour under pixel before being overwritten
} Molecule;

class Preferences;

class WaterEngine
{
public:
	Bitmap *scrimage;
	bool densities;
	bool filter;
	u32 mols;
	Molecule *mol;
	Preferences *prefs;
	// These are the magic colours which change the automata rules
	u32 pumpUscrcol;
	u32 pumpLscrcol;
	u32 pumpRscrcol;
	u32 filtAscrcol;
	u32 filtRscrcol;
	u32 filtBscrcol;

public:
	WaterEngine::WaterEngine(u32 _mols, Bitmap *bmp, Preferences *_prefs);

	WaterEngine::~WaterEngine();

	void WaterEngine::update();
private:
	ENGOPTS bool ENGOPTS2 WaterEngine::_testdown(u32 *curoffset, const u32 width, const u32 height, u32 oldscrcol, bool light) const;
	ENGOPTS bool ENGOPTS2 WaterEngine::_testlr(u32 *curoffset, const u32 width, const u32 height, u32 oldscrcol, bool light);
	ENGOPTS bool ENGOPTS2 WaterEngine::_testup(u32 *curoffset, const u32 width, const u32 height, u32 oldscrcol, bool light) const;
public:
	void WaterEngine::render(bool plot);

	void WaterEngine::palettechanged();

	void WaterEngine::setSize(u32 w, u32 h);

	void WaterEngine::setNoOfMols(u32 newmols, int col);

	bool WaterEngine::load(HANDLE inh);
	void WaterEngine::save(HANDLE outh, bool autoset);
};