/* common.cpp
Implements useful classes
(C) 2000 N. Douglas
12th April 2000
*/

#include "common.h"
#include <stdio.h>

#ifdef _DEBUG
// Overload new and delete to zero memory on alloc
void *operator new(size_t amount)
{
	void *t;
	t=malloc(amount);
	memset(t, 0, amount);
	return t;
}

void operator delete(void *mem)
{
	free(mem);
}
#endif

Error::Error(char *file, int line, const char *message)
{
	module=file;
	lineno=(u32) line;
	if(message)
	{
		external=false;
		strcpy(msg, message);
		return;
	}
#ifdef WIN32
	else
	{
		// Is a Win32 error - decode
		win32code=GetLastError();
		if(FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, win32code,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), msg, 256, NULL)==0)
		{
			// FormatMessage failed, panic!
			throw "FATAL: Win32 Error message conversion failed!";
		}
		external=true;
		return;
	}
#endif
}

void Error::toString(char *buff, int len)
{
	if(external)
		strcpy(buff, "External error: ");
	else
		strcpy(buff, "Error: ");
	strcat(buff, msg);
#ifdef WIN32
	if(external) sprintf(strchr(buff,0), "(win32 code 0x%x) ", win32code);
#endif
	sprintf(strchr(buff,0), "- thrown at line %d in file %s", lineno, module);
	if((int) strlen(buff)>=len)
	{
		throw "FATAL: Error to string conversion buffer overflowed!";
	}
}

void Error::showAsMsg()
{
	MessageBox(NULL, msg, "Message from Flow", MB_OK);
}
