/* win32bitmap.h
Defines the Bitmap class
(C) 2000 N. Douglas
13th April 2000
*/

#include "common.h"

#ifdef WIN32
#include "ddraw.h"
#else
#error This is suitable for Win32 builds only
#endif

// The Bitmap superclass encapsulates all the DirectX stuff to do
// bitmap ops using the hardware where possible
// 
// One then derives child classes which access the bitmap in a run-
// time fashion which is much quicker than checking the bpp and
// casting for every access

class Bitmap
{
	friend class ScreenImage8;
	friend class ScreenImage16;
	friend class ScreenImage32;
public:
	u32 width, height, bpp;
	bool fullscreen;
	static bool GDIOnly;
protected:
	HWND hwnd;
	void *bmpaddr;
private:
	u32 refreshdelay;           // ms to wait between screen refreshes
	HANDLE refreshevent;
	u32 primarybpp;
	bool NTmodeswitch;
	PALETTEENTRY *paltemplate;
	HDC bmpDC;
	LPDIRECTDRAW ddH;
	LPDIRECTDRAWSURFACE ddsP;   // DirectDraw primary surface
	LPDIRECTDRAWCLIPPER ddcP;	// Primary surface clipper
	LPDIRECTDRAWPALETTE ddpP;	// Primary surface palette
	LPDIRECTDRAWSURFACE ddsB;   // DirectDraw back buffer surface
	DDSURFACEDESC ddsd;

private:
	bool Bitmap::DDErrH(HRESULT ddrval, int lineno);

public:
	Bitmap::Bitmap();
private:
	void Bitmap::Init();
	LPDIRECTDRAWSURFACE Bitmap::createBBSurface(u32 w, u32 h);
public:
	Bitmap::~Bitmap();

	void Bitmap::setSize(u32 w, u32 h);

	void Bitmap::paint(HDC dc, s32 x, s32 y, u32 scale);

	HDC Bitmap::getDC();
	void Bitmap::releaseDC();

	void *Bitmap::getAddr();
	void Bitmap::releaseAddr();

	void Bitmap::modeChanged();
	void Bitmap::installPalette();
	void Bitmap::setFullScreen(bool);

	u32 Bitmap::RGBtoPhysical(u32 rgb);

	void Bitmap::load(HANDLE inh);
	void Bitmap::save(HANDLE outh);

public:
	virtual void writepixel(u32, u32)=0;
	virtual u32 readpixel(u32)=0;

};

class ScreenImage8 : public Bitmap
{

public:
	ScreenImage8(HWND hw, u32 w, u32 h)
	{
		hwnd=hw;
		width=w;
		height=h;
		bpp=8;
		Init();
	}

	~ScreenImage8()
	{
		hwnd=NULL;
		width=0;
		height=0;
		bpp=0;
	}

	inline void writepixel(u32 offset, u32 col)
	{
		((u8 *) bmpaddr)[offset]=(u8) col;
	}

	inline u32 readpixel(u32 offset)
	{
		return ((u8 *) bmpaddr)[offset];
	}
};

class ScreenImage16 : public Bitmap
{

public:
	ScreenImage16(HWND hw, u32 w, u32 h)
	{
		hwnd=hw;
		width=w;
		height=h;
		bpp=16;
		Init();
	}

	~ScreenImage16()
	{
		hwnd=NULL;
		width=0;
		height=0;
		bpp=0;
	}

	inline void writepixel(u32 offset, u32 col)
	{
		((u16 *) bmpaddr)[offset]=(u16) col;
	}

	inline u32 readpixel(u32 offset)
	{
		return ((u16 *) bmpaddr)[offset];
	}
};

class ScreenImage32 : public Bitmap
{

public:
	ScreenImage32(HWND hw, u32 w, u32 h)
	{
		hwnd=hw;
		width=w;
		height=h;
		bpp=32;
		Init();
	}

	~ScreenImage32()
	{
		hwnd=NULL;
		width=0;
		height=0;
		bpp=0;
	}

	inline void writepixel(u32 offset, u32 col)
	{
		((u32 *) bmpaddr)[offset]=col;
	}

	inline u32 readpixel(u32 offset)
	{
		return ((u32 *) bmpaddr)[offset];
	}
};
