/* water.c
Animates some water
(C) 2000 N. Douglas
9th March 2000
*/

#include <stdio.h>
#include "Game.h"

// Include windows, COM, ShlObj and resources
#include "windows.h"
#include "objbase.h"
#include "shlobj.h"
#include "resource.h"

#define GAMES 32

HINSTANCE hinst;
bool debug;
static UINT globalInterFlowMsg;

#define REGACC(basekeyh, keyname, defval) \
	if(remove) \
	{ \
		TEST_WIN32ERR(ERROR_SUCCESS!=RegDeleteKey(basekeyh, keyname)); \
	} \
	else \
	{ \
		HKEY tkeyh; \
		DWORD disp; \
		TEST_WIN32ERR(ERROR_SUCCESS!=RegCreateKeyEx(basekeyh, keyname, 0, \
			NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &tkeyh, &disp)); \
		TEST_WIN32ERR(ERROR_SUCCESS!=RegSetValueEx(tkeyh, NULL, 0, REG_SZ, \
			(BYTE *) defval, strlen(defval)+1)); \
		TEST_WIN32ERR(ERROR_SUCCESS!=RegCloseKey(tkeyh)); \
	}

static void InstallFileTypes(bool remove, bool startmenu, bool desktop)
{
	HKEY keyh;
	DWORD disp;
	int len;
	char exepath[_MAX_PATH], *exepathend;

	TEST_WIN32ERR(!(len=GetModuleFileName(NULL, exepath, _MAX_PATH)));
	exepathend=exepath+len;

	// Add uninstall entry
	try
	{
		TEST_WIN32ERR(ERROR_SUCCESS!=RegOpenKeyEx(HKEY_LOCAL_MACHINE,
			"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall", 0,
			KEY_ALL_ACCESS, &keyh));
		if(remove)
		{
			TEST_WIN32ERR(ERROR_SUCCESS!=RegDeleteKey(keyh, "FlowGame"));
		}
		else
		{
			HKEY mykeyh;
			TEST_WIN32ERR(ERROR_SUCCESS!=RegCreateKeyEx(keyh, "FlowGame", 0,
				NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &mykeyh, &disp));
			TEST_WIN32ERR(ERROR_SUCCESS!=RegSetValueEx(mykeyh, "DisplayName", 0, REG_SZ,
				(BYTE *) "Flow Game (remove only)", 24));
			strcpy(exepathend," -uninstall");
			TEST_WIN32ERR(ERROR_SUCCESS!=RegSetValueEx(mykeyh, "UninstallString", 0, REG_SZ,
				(BYTE *) exepath, strlen(exepath)+1));
			TEST_WIN32ERR(ERROR_SUCCESS!=RegCloseKey(mykeyh));
		}
		TEST_WIN32ERR(ERROR_SUCCESS!=RegCloseKey(keyh));
	}
	catch(...)
	{
	}

	try
	{
		// Check extension
		REGACC(HKEY_CLASSES_ROOT, ".flowgame", "Flow.FlowGame");

		// Check extension decode
		TEST_WIN32ERR(ERROR_SUCCESS!=RegCreateKeyEx(HKEY_CLASSES_ROOT, "Flow.FlowGame", 0,
			NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &keyh, &disp));
		if(!remove && disp==REG_CREATED_NEW_KEY)
		{
			TEST_WIN32ERR(ERROR_SUCCESS!=RegSetValueEx(keyh, NULL, 0, REG_SZ,
				(BYTE *) "Flow Saved Game", 16));
		}
		strcpy(exepathend, ",1");
		REGACC(keyh, "DefaultIcon", exepath);

		// Now shell\open\command
		{
			HKEY shellh,openh;
			TEST_WIN32ERR(ERROR_SUCCESS!=RegCreateKeyEx(keyh, "shell", 0,
				NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &shellh, &disp));
			TEST_WIN32ERR(ERROR_SUCCESS!=RegCreateKeyEx(shellh, "open", 0,
				NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &openh, &disp));
			strcpy(exepathend, " -load \"%1\"");
			REGACC(openh, "command", exepath);
			TEST_WIN32ERR(ERROR_SUCCESS!=RegCloseKey(openh));
			if(remove) TEST_WIN32ERR(ERROR_SUCCESS!=RegDeleteKey(shellh, "open"));
			TEST_WIN32ERR(ERROR_SUCCESS!=RegCloseKey(shellh));
		}
		if(remove) TEST_WIN32ERR(ERROR_SUCCESS!=RegDeleteKey(keyh, "shell"));
		TEST_WIN32ERR(ERROR_SUCCESS!=RegCloseKey(keyh));
		if(remove) TEST_WIN32ERR(ERROR_SUCCESS!=RegDeleteKey(HKEY_CLASSES_ROOT, "Flow.FlowGame"));
	}
	catch(...)
	{
	}

	// Add item to Start Menu
	*exepathend=0;
	{
    IShellLink* pLink;
    IPersistFile* pPersistFile;
    LPMALLOC      ShellMalloc;
    LPITEMIDLIST  ProgramsPidl, DesktopPidl;
    char ProgramsDir[MAX_PATH];
    char DesktopDir[MAX_PATH];

    // We are going to create a pidl, and it will need to be
    // freed by the shell mallocator. Get the shell mallocator
    // object using API SHGetMalloc function.
    if(FAILED(SHGetMalloc(&ShellMalloc)))
	{
		DECLARE_ERROR(e, "Shell COM Error 1");
		throw e;
	}
    // use the API to get a pidl for the directory
    if(FAILED(SHGetSpecialFolderLocation(NULL,
                                         CSIDL_PROGRAMS,
                                         &ProgramsPidl)))
	{
		DECLARE_ERROR(e, "Shell COM Error 2");
		throw e;
	}
    if(!SHGetPathFromIDList(ProgramsPidl, ProgramsDir))
    {
		DECLARE_ERROR(e, "Shell COM Error 3");

        ShellMalloc->Free(ProgramsPidl);
        ShellMalloc->Release();
		throw e;
    }
    ShellMalloc->Free(ProgramsPidl);

    // use the API to get a pidl for the directory
    if(FAILED(SHGetSpecialFolderLocation(NULL,
                                         CSIDL_DESKTOP,
                                         &DesktopPidl)))
	{
		DECLARE_ERROR(e, "Shell COM Error 4");
		throw e;
	}
    if(!SHGetPathFromIDList(DesktopPidl, DesktopDir))
    {
		DECLARE_ERROR(e, "Shell COM Error 5");

        ShellMalloc->Free(DesktopPidl);
        ShellMalloc->Release();
		throw e;
    }
	ShellMalloc->Free(DesktopPidl);
    ShellMalloc->Release();

	strcat(ProgramsDir, "\\Flow.lnk");
	strcat(DesktopDir, "\\Flow.lnk");
    if(!remove && SUCCEEDED(CoInitialize(NULL)))
    {
		if(SUCCEEDED(CoCreateInstance(CLSID_ShellLink, NULL,
		                              CLSCTX_INPROC_SERVER,
		                              IID_IShellLink, (void **) &pLink)))
		{
		    pLink->SetPath(exepath);
		    pLink->SetShowCmd(SW_SHOW);
			*strrchr(exepath, '\\')=0;
			pLink->SetWorkingDirectory(exepath);
	
			if(startmenu)
			{
	            if(SUCCEEDED(pLink->QueryInterface(IID_IPersistFile,
	                                               (void **)&pPersistFile)))
	            {
		            WORD wsz[MAX_PATH]; 
		            // Ensure that the string is unicode. 
		            MultiByteToWideChar(CP_ACP, 0, ProgramsDir, -1, 
		                wsz, MAX_PATH); 
	                pPersistFile->Save(wsz, TRUE);
	                pPersistFile->Release();
	            }
			}
			if(desktop)
			{
	            if(SUCCEEDED(pLink->QueryInterface(IID_IPersistFile,
	                                               (void **)&pPersistFile)))
	            {
		            WORD wsz[MAX_PATH]; 
		            // Ensure that the string is unicode. 
		            MultiByteToWideChar(CP_ACP, 0, DesktopDir, -1, 
		                wsz, MAX_PATH); 
	                pPersistFile->Save(wsz, TRUE);
	                pPersistFile->Release();
	            }
			}
            pLink->Release();
        }
        CoUninitialize();
    }
	if(remove)
	{
		DeleteFile(ProgramsDir);
		DeleteFile(DesktopDir);
	}
	}
}

static BOOL CALLBACK InstallDialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{ 
    switch (uMsg)
	{ 
	case WM_INITDIALOG:
		{
			TEST_WIN32ERR(!CheckDlgButton(hwndDlg, IDC_ADD_SM_SHRTCUT, BST_CHECKED));
			TEST_WIN32ERR(!CheckDlgButton(hwndDlg, IDC_ADD_DESK_SHRTCUT, BST_CHECKED));
			TEST_WIN32ERR(!CheckDlgButton(hwndDlg, IDC_PLAY_GAME_NOW, BST_CHECKED));
			return TRUE;
		}
        case WM_COMMAND: 
            switch (LOWORD(wParam))
			{ 
				case IDCANCEL:
					{
						EndDialog(hwndDlg, 0);
						return TRUE;
					}
                case IDOK:
					{
						int ret=0;
						if(BST_CHECKED==IsDlgButtonChecked(hwndDlg, IDC_ADD_SM_SHRTCUT)) ret|=0x4;
						if(BST_CHECKED==IsDlgButtonChecked(hwndDlg, IDC_ADD_DESK_SHRTCUT)) ret|=0x2;
						if(BST_CHECKED==IsDlgButtonChecked(hwndDlg, IDC_PLAY_GAME_NOW)) ret|=0x1;
						EndDialog(hwndDlg, ret); 
						return TRUE;
					}
            } 
    } 
    return FALSE; 
} 

LRESULT CALLBACK WndProcStub( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	Game *g;
	// Redirect to our class based window handler
	g=(Game *) GetWindowLong(hwnd, 0);
	return (g) ? g->wndproc(hwnd, uMsg, wParam, lParam) : DefWindowProc(hwnd, uMsg, wParam, lParam);
}

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, 
    LPSTR lpszCmdLine, int nCmdShow) 
{ 
    MSG msg; 
    WNDCLASSEX wc;
	LPTSTR CmdLine;
	char loadpath[_MAX_PATH];
 
	try {
    // Register the window class for the main window. 
 
    if (!hPrevInstance) 
    { 
		wc.cbSize=sizeof(WNDCLASSEX);
        wc.style = CS_CLASSDC | CS_BYTEALIGNCLIENT | CS_HREDRAW | CS_VREDRAW ; 
        wc.lpfnWndProc = (WNDPROC) WndProcStub; 
        wc.cbClsExtra = 0; 
        wc.cbWndExtra = sizeof(Game *); 
        wc.hInstance = hInstance; 
        wc.hIcon = LoadIcon(hInstance, 
            MAKEINTRESOURCE(IDI_MAINICON)); 
        wc.hCursor = LoadCursor((HINSTANCE) NULL, 
            IDC_ARROW); 
        wc.hbrBackground = (HBRUSH) GetStockObject(GRAY_BRUSH); 
        wc.lpszMenuName =  MAKEINTRESOURCE(IDR_MAINMENU); 
        wc.lpszClassName = "MainWndClass";
		wc.hIconSm = NULL;
 
        TEST_WIN32ERR(!RegisterClassEx(&wc)) ;
    } 
 
    hinst = hInstance;  // save instance handle 

	// Decide what command line wants & update registry
	// HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\<appname>
	CmdLine=GetCommandLine();
	if(strstr(CmdLine, "-uninstall")!=0)
	{
		// Remove file types
		if(MessageBox(NULL, "Are you sure you wish to uninstall Flow?", "Question from Flow", MB_ICONQUESTION | MB_YESNO)==IDYES)
		{
			InstallFileTypes(true, false, false);
			// Try deleting ourselves
			TEST_WIN32ERR(!(GetModuleFileName(NULL, loadpath, _MAX_PATH)));
			*strrchr(loadpath, '\\')=0;
			if(!MoveFileEx(loadpath, NULL, MOVEFILE_DELAY_UNTIL_REBOOT))
			{
				char temp[_MAX_PATH]="Unable to delete program files (in ";
				strcat(temp, loadpath);
				strcat(temp, ") - you will have to do this yourself!");
				MessageBox(NULL, temp, "Message from Flow", MB_ICONWARNING | MB_OK);
			}
			else
				MessageBox(NULL, "Program is uninstalled. Program files will be deleted upon next reboot", "Message from Flow", MB_ICONINFORMATION | MB_OK);
		}
		return 0;
	}
	else
	{
		bool exit=false, startmenu=false, desktop=false;
		if(strstr(CmdLine, "-nogame")!=0)
		{
			int ret;
			// We have just been installed. Ask if we install shortcuts
			ret=DialogBox(hinst, MAKEINTRESOURCE(IDD_INSTALLBOX), NULL, (DLGPROC) InstallDialogProc);
			TEST_WIN32ERR(ret==-1);
			if(ret & 0x1) exit=false;	else exit=true;
			if(ret & 0x2) desktop=true;	else desktop=false;
			if(ret & 0x4) startmenu=true; else startmenu=false;
		}
		// Install file types
		InstallFileTypes(false, startmenu, desktop);
		if(exit) return 0;
	}
	if(strstr(CmdLine, "-debug")!=0)
	{
		// Debug mode requested
		debug=true;
	}
	else debug=false;

	Game *game[GAMES];
	memset(game, 0, GAMES*sizeof(Game *));
	
	// See if we need to start a new game
	globalInterFlowMsg=RegisterWindowMessage("Flow_InterFlowCommMsg");
	{
		char *filep;
		filep=strstr(CmdLine, "-load");
		if(filep)
		{
			while(*filep!=' ') filep++;
			if(*(++filep)=='"')
			{
				*strrchr(filep++,'"')=0;
			}
		}
		// Open a window, whether there's a file to load or not
		PostMessage(NULL, WM_NEWGAMEREQ, nCmdShow, (LPARAM) filep);
	}

	// Should we only use GDI?
	if(strstr(CmdLine, "-gdi"))
		Bitmap::GDIOnly=true;

	// Configure the multimedia timer
	{
		TIMECAPS tc;
		if(TIMERR_NOERROR!=timeGetDevCaps(&tc, sizeof(tc)))
		{
			DECLARE_ERROR(e, "Multimedia timer not available");
			throw e;
		}
		if(TIMERR_NOERROR!=timeBeginPeriod(min(max(tc.wPeriodMin, 1), tc.wPeriodMax)))
		{
			DECLARE_ERROR(e, "Multimedia timer will not set resolution");
			throw e;
		}
	}

    // Start the message loop. 
    while(1)
    {
		BOOL ret;
		ret=GetMessage( &msg, NULL, 0, 0 );
		TEST_WIN32ERR(ret<0);
		if(ret>0)
		{
			TranslateMessage(&msg);
			if(msg.hwnd)
				DispatchMessage(&msg);
			else if(msg.message==WM_CLOSEGAMEREQ)		// Game object wishes to die (lParam=ptr of Game object)
			{
				Game *g;
				int activegames=0;
				// Delete the Game object specified by lParam
				g=(Game *) msg.lParam;
				for(int n=0; n<GAMES; n++)
				{
					if(g==game[n])
					{
						delete game[n];
						game[n]=0;
					}
					if(game[n]!=0) activegames++;
				}
				// If all games deleted, quit
				if(!activegames) PostQuitMessage(0);
			}
			else if(msg.message==WM_NEWGAMEREQ)			// New game requested (wParam=initial show state, lParam=file path)
			{
				for(int n=0; n<GAMES; n++)
				{
					if(!game[n])
					{
						if(!msg.lParam)
						{
							char temp[32];
							sprintf(temp,"FlowGame%d", n);
							game[n]=new Game(hInstance, GetCurrentThreadId(), temp, msg.wParam);
						}
						else
						{
							game[n]=new Game(hInstance, GetCurrentThreadId(), (char *) msg.lParam, msg.wParam);
							// Now trigger loading
							while(!PostThreadMessage(game[n]->mythreadID, WM_COMMAND, IDM_LOADGAME, 0))
								Sleep(0);
						}
						break;
					}
				}
			}
		}
		else
		{
			// Delete all game objects (this lets them kill off their threads)
			for(int n=0; n<GAMES; n++)
			{
				if(game[n])
				{
					delete game[n];
					game[n]=0;
				}
			}
			break;
		}
	}
	}
	catch(Error e)
	{
		char temp[256];
		e.toString(temp, 256);
		MessageBox(NULL, temp, "Error:", MB_OK);
		return 1;
	}
	catch(char *e)
	{
		MessageBox(NULL, e, "FATAL ERROR:", MB_OK);
		return 1;
	}
 
    // Return the exit code to Windows. 
 
    return msg.wParam; 
} 
 